/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.basis;

import java.util.HashMap;
import java.util.Map;

/**
 * This class represents an entity identifier.
 * 
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public final class EntityType implements Comparable<EntityType> {
  // The cache of the entity types.
  private static Map<String, EntityType> cache = new HashMap<>();

  /**
   * Returns the entity type with the given identifier.
   * 
   * @param id the entity type identifier
   * @return the entity type.
   */
  public static EntityType get(final String id) {
    EntityType result = cache.get(id);

    if (result == null) {
      cache.put(id, result = new EntityType(id));
    }

    return result;
  }

  // The entity type identifier.
  private String id;

  /**
   * Constructs an entity type.
   * 
   * @param id the entity type identifier.
   */
  private EntityType(final String id) {
    this.id = id;
  }

  /**
   * Returns the entity type identifier.
   * 
   * @return the entity type identifier.
   */
  public String getId() {
    return id;
  }

  @Override
  public String toString() {
    return id;
  }

  @Override
  public boolean equals(final Object obj) {
    if (!(obj instanceof EntityType)) {
      return false;
    }

    final EntityType rhs = (EntityType) obj;
    return id.equals(rhs.id);
  }

  @Override
  public int hashCode() {
    return id.hashCode();
  }

  @Override
  public int compareTo(final EntityType rhs) {
    return id.compareTo(rhs.id);
  }
}
