/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.basis;

import java.util.ArrayList;
import java.util.List;

/**
 * This class represents a collection of file names.
 * 
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public class FileNames extends Entity {

  // The file names' extension (file type).
  private String type;
  // The list of file names.
  private List<String> fileNames = new ArrayList<>();

  /**
   * Constructs a file names entity for a given type.
   * 
   * @param type the files' type (extension).
   */
  public FileNames(final String type) {
    super(EntityType.get(type));

    this.type = type;
  }

  /**
   * Returns the file names' extension (file type).
   *
   * @return the file type.
   */
  public String getType() {
    return type;
  }

  /**
   * Adds a given file name to the list.
   * 
   * @param fileName the file name to be added.
   */
  public void add(final String fileName) {
    fileNames.add(fileName);
  }

  /**
   * Returns the file names (as a list).
   * 
   * @return the file names.
   */
  public List<String> getList() {
    return fileNames;
  }

  /**
   * Returns the file names (as an array).
   * 
   * @return the file names.
   */
  public String[] getArray() {
    return fileNames.toArray(new String[] {});
  }

  @Override
  public void add(final Entity entity) {
    FileNames r = (FileNames) entity;
    fileNames.addAll(r.fileNames);
  }

  @Override
  public String toString() {
    return fileNames.toString();
  }
}
