/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.basis;

import java.util.LinkedList;
import java.util.List;

import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

/**
 * This class represents a specification of command line parameters.
 * 
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public final class Parameters {
  // The list of supported parameters.
  private List<Parameter> parameters = new LinkedList<>();
  // The list of supported parameters in the Apache Commons CLI format.
  private Options options = new Options();

  /**
   * Returns the list of supported parameters.
   * 
   * @return the supported parameters.
   */
  public List<Parameter> getParameters() {
    return parameters;
  }

  /**
   * Returns the list of supported parameters in the Apache Commons CLI format.
   * 
   * @return the supported parameters.
   */
  public Options getOptions() {
    return options;
  }

  /**
   * Adds a given option to the specification.
   * 
   * @param parameter the option to be added.
   */
  public void addParameter(final Parameter parameter) {
    parameters.add(parameter);
    options.addOption(parameter.getOption());
  }

  /**
   * Parses the command line.
   * 
   * @param args the command line.
   * @return the arguments
   * @throws ParseException the parse exception.
   */
  public Arguments parseCommandLine(final String[] args) throws ParseException {
    return new Arguments(this, args);
  }

  /**
   * Prints the help message.
   * 
   * @param format the tool execution format.
   */
  public void help(final String format) {
    HelpFormatter formatter = new HelpFormatter();
    formatter.printHelp(80, format, "", options, "");
  }
}
