/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
package ru.ispras.retrascope.basis;

import java.nio.file.Path;
import java.util.HashSet;
import java.util.Set;

/**
 * The representation of a VHDL testbench.
 *
 * @author <a href="mailto:i.melnichenko@deltasolutions.ru">Igor Melnichenko</a>
 */
public class VhdlTestbench extends FileNames {
  /**
   * The entity type ID of all {@code VhdlTestbench} instances.
   *
   * @see ru.ispras.retrascope.basis.EntityType#get(String)
   * @see Entity#Entity(ru.ispras.retrascope.basis.EntityType)
   */
  public static final String ENTITY_TYPE_ID = ".vhd".intern();

  private final Set<Path> additionalFiles;

  /**
   * Constructs a new VHDL testbench with the specified testbench file and additional files (for
   * example, test pattern and library files).
   *
   * @param extensionlessTestbenchFileName a name of the testbench file without extension
   * @param additionalFiles a set containing additional files of the testbench
   */
  public VhdlTestbench(String extensionlessTestbenchFileName, Set<Path> additionalFiles) {
    super(VhdlTestbench.ENTITY_TYPE_ID);
    this.add(extensionlessTestbenchFileName);
    this.additionalFiles = new HashSet<>(additionalFiles);
  }

  /**
   * Returns additional files (for example, test pattern and library files) related to this
   * testbench.
   *
   * @return a set of the additional files
   */
  public Set<Path> getAdditionalFiles() {
    return new HashSet<Path>(this.additionalFiles);
  }
}
