/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ru.ispras.retrascope.engine.cfg;

import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.retrascope.model.cfg.CfgVisitor;
import ru.ispras.retrascope.model.cfg.CfgWalkerMode;

/**
 * Object that stores visitor {@link ru.ispras.retrascope.model.cfg.CfgVisitor}
 * and walking {@link ru.ispras.retrascope.model.cfg.CfgWalkerMode} mode
 * for control flow graph walker {@link ru.ispras.retrascope.model.cfg.CfgWalker}.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class CfgVisitorConfig {

  /* The visitor. */
  private final CfgVisitor visitor;

  /* The walking mode. */
  private final CfgWalkerMode walkerMode;

  /**
   * Constructs configuration with the specified visitor and walker mode.
   *
   * @param visitor control flow graph visitor
   * @param mode control flow graph walker mode
   *
   * @throws java.lang.NullPointerException when any of arguments is <code>null</code>.
   */
  public CfgVisitorConfig(CfgVisitor visitor, CfgWalkerMode mode) {
    InvariantChecks.checkNotNull(visitor);
    InvariantChecks.checkNotNull(mode);

    this.visitor = visitor;
    this.walkerMode = mode;
  }

  /**
   * Returns control flow graph visitor.
   *
   * @return visitor
   */
  public CfgVisitor getVisitor() { return this.visitor; }

  /**
   * Returns control flow graph walker mode.
   *
   * @return walker mode
   */
  public CfgWalkerMode getWalkerMode() { return this.walkerMode; }
}
