/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ru.ispras.retrascope.engine.cgaa.printer.graphml;

import org.apache.commons.cli.ParseException;
import ru.ispras.retrascope.basis.Arguments;
import ru.ispras.retrascope.basis.Engine;
import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.engine.cfg.printer.graphml.CfgGraphMlPrinter;
import ru.ispras.retrascope.model.cfg.CfgModel;
import ru.ispras.retrascope.model.cgaa.CgaaModel;

import java.util.Map;

/**
 * Clocked guarded atomic actions model {@link ru.ispras.retrascope.model.cgaa.CgaaModel}
 * printer to GraphML format.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class CgaaGraphMlPrinter extends Engine {

  /* The engine identifier. */
  public static final String ENGINE_ID = "cgaa-graphml-printer";

  /* The engine output entity type. */
  public static final EntityType OUT_TYPE = EntityType.get("cgaa-graphml");

  /* Target GraphML file default name. */
  private static final String CGAA_GRAPHML_FILE_NAME = "cgaa-model.graphml";

  /**
   * Constructs GraphML printer object.
   */
  public CgaaGraphMlPrinter() {
    super(CgaaGraphMlPrinter.ENGINE_ID);
    this.addInputType(CgaaModel.TYPE);
    this.setOutputType(CgaaGraphMlPrinter.OUT_TYPE);
    this.addParameter(CfgGraphMlPrinter.GRAPHML_PARAM);
  }

  @Override
  public Entity start(Map<EntityType, Entity> inputs) {

    /* Extract the embedded control flow graph model. */
    final CgaaModel cgaaModel = (CgaaModel) inputs.get(CgaaModel.TYPE);
    inputs.remove(CgaaModel.TYPE);
    inputs.put(CfgModel.TYPE, cgaaModel.getModel());

    /* Extract target file name from engine inputs. */
    final Arguments arguments = (Arguments)inputs.get(Arguments.TYPE);
    String fileName = null;
    Arguments engineArgs;
    try {
      engineArgs = parseCommandLine(arguments.getCommandLine());
      fileName = engineArgs.getValue(CfgGraphMlPrinter.GRAPHML_PARAM, CGAA_GRAPHML_FILE_NAME);
    } catch (ParseException e) {
      e.printStackTrace();
    }

    /* Run the control flow graph model GraphML printer on the extracted model. */
    CfgGraphMlPrinter printer = new CfgGraphMlPrinter();
    printer.initGraphMlVisitor(fileName);
    return printer.start(inputs);
  }
}
