/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.conflict.printer.xml;

import java.nio.file.Paths;
import java.util.Map;
import java.util.logging.Logger;

import javax.xml.bind.JAXB;

import org.apache.commons.cli.ParseException;

import ru.ispras.retrascope.basis.Arguments;
import ru.ispras.retrascope.basis.Engine;
import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.basis.FileNames;
import ru.ispras.retrascope.basis.Parameter;
import ru.ispras.retrascope.engine.efsm.extractor.conflict.EfsmConflictExtractor;
import ru.ispras.retrascope.result.conflict.ConflictModel;
import ru.ispras.retrascope.util.Log;
import ru.ispras.retrascope.util.LogLevel;

/**
 * This engine prints conflicts acquired by the {@link EfsmConflictExtractor} to an XML file.
 * 
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public class ConflictXmlPrinter extends Engine {

  /* The engine identifier. */
  public static final String ENGINE_ID = "conflict-xml-printer";
  
  /* The engine output entity type. */
  public static final EntityType OUT_TYPE = EntityType.get("conflicts-xml");
  
  /* The command-line parameter. */
  public static final Parameter FILE_NAME = new Parameter("output-file", true,
      "File name to dump XML representation of resulting conflicts");

  private final Logger logger = Log.getLogger(this.getClass());
  private final String LOG_HEADER = "Conflict XML Printer: ";

  public ConflictXmlPrinter() {
    super(ENGINE_ID);
    addInputType(ConflictModel.TYPE);
    setOutputType(OUT_TYPE);
    addParameter(FILE_NAME);
  }

  @Override
  public Entity start(Map<EntityType, Entity> inputs) {

    final FileNames fileNames = new FileNames("xml");

    final ConflictModel conflicts = (ConflictModel) inputs.get(ConflictModel.TYPE);

    try {

      final Arguments arguments =
          this.parseCommandLine(((Arguments) inputs.get(Arguments.TYPE)).getCommandLine());

      final String xmlFile = arguments.getValue(FILE_NAME, "conflicts.xml");

      // Storing conflicts to *.xml file.
      if (!conflicts.getConflicts().isEmpty()) {
        JAXB.marshal(conflicts, Paths.get(xmlFile).toFile());
        logger.log(LogLevel.INFO, LOG_HEADER + "XML file created: " + xmlFile);
        fileNames.add(xmlFile);
      }

    } catch (ParseException | NumberFormatException e1) {
      logger.log(LogLevel.SEVERE, LOG_HEADER, e1);
    }

    return fileNames;
  }

}
