/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.efsm.extractor.conflict.searcher;

import java.util.List;

import ru.ispras.retrascope.model.efsm.GuardedAction;
import ru.ispras.retrascope.model.efsm.Efsm;
import ru.ispras.retrascope.model.efsm.EfsmTransition;

/**
 * Base class for different EFSM searching algorithms.
 * 
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public interface EfsmPathSearcher {

  /**
   * Used to initialize the searcher.
   * 
   * @param efsm - the {@link Efsm} to examine;
   * @param startTransition - initial point of the algorithm ({@link EfsmTransition});
   * @param condition - the {@link SearchCondition} that is required to stop the search.
   */
  public abstract void init(Efsm efsm, EfsmTransition startTransition, SearchCondition condition);

  /**
   * Shows whether the algorithm can proceed and try to obtain alternative results.
   * 
   * @return {@code true} if the algorithm can proceed, {@code false} otherwise.
   */
  public abstract boolean hasNext();

  /**
   * Performs a search until condition is met or the end is reached.
   */
  public abstract void next();

  /**
   * Shows whether the result has been obtained during the search.
   * 
   * @return {@code true} if the path has been found, {@code false} otherwise.
   */
  public abstract boolean hasValue();

  /**
   * Returns the path if it has been found.
   * 
   * @return A {@link List} of {@link GuardedAction}
   */
  public abstract List<GuardedAction> value();

}
