/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.efsm.extractor.conflict.searcher;

/**
 * The interface that represents the conditions which limit the search algorithm.
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public interface SearchCondition {

  /**
   * Checks whether the stop condition is reached.
   * @param obj - the object to be checked.
   * @return {@code true} if the condition is reached, {@code false} otherwise.
   */
  boolean conditionMet(Object obj);

  /**
   * Checks whether the depth limit is reached.
   * @param depth - the desirable depth of the search.
   * @return {@code true} if the limit is reached, {@code false} otherwise.
   */
  boolean limitReached(int depth);
}
