/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
package ru.ispras.retrascope.engine.efsm.generator.test.heuristic.pathfinders.dfs;

import java.util.List;
import java.util.concurrent.Executors;

import ru.ispras.fortress.expression.NodeOperation;
import ru.ispras.retrascope.engine.efsm.generator.test.heuristic.pathfinders.EfsmPathfinder;
import ru.ispras.retrascope.model.efsm.Efsm;
import ru.ispras.retrascope.model.efsm.EfsmTransition;

public class DepthFirstSearchPathfinder extends EfsmPathfinder implements AutoCloseable {
  private final DepthFirstSearchTraverser traverser;

  public static int getCycleNumberForAnnotatedInput(String annotatedName) {
    return DepthFirstSearchTraverser.getCycleNumberForAnnotatedInput(annotatedName);
  }

  public static String deannotateName(String annotatedName) {
    return DepthFirstSearchTraverser.deannotateName(annotatedName);
  }

  public DepthFirstSearchPathfinder(Efsm efsm, EfsmTransition transition, int loopLimit) {
    this.traverser = new DepthFirstSearchTraverser(efsm, transition, loopLimit);
  }

  @Override
  public List<EfsmTransition> getNext() {
    if (!this.traverser.isFinished()) {
      if (this.traverser.isStarted()) {
        this.traverser.getNext();
      } else {
        Executors.newSingleThreadExecutor().execute(this.traverser);
      }

      while (!this.traverser.isFinished()) {
        try {
          if (this.traverser.isNextReady()) {
            return this.traverser.getPath();
          }

          Thread.sleep(10);
        } catch (InterruptedException e) {
          this.traverser.finish();

          break;
        }
      }
    }

    return null;
  }

  public List<NodeOperation> getConstraints() {
    return this.traverser.getConstraints();
  }

  @Override
  public void close() throws Exception {
    this.traverser.finish();
  }
}
