/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.efsm.printer.graphml;

import java.util.Map;

import org.apache.commons.cli.ParseException;

import ru.ispras.retrascope.basis.Arguments;
import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.basis.FileNames;
import ru.ispras.retrascope.basis.Parameter;
import ru.ispras.retrascope.engine.efsm.EfsmEngine;
import ru.ispras.retrascope.model.efsm.EfsmModel;
import ru.ispras.retrascope.model.efsm.EfsmModelWalker;

/**
 * EFSM printer to GraphML format.
 * 
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */

public class EfsmGraphMlPrinter extends EfsmEngine {

  /** The engine identifier. */
  public static final String ENGINE_ID = "efsm-graphml-printer";

  /** The engine output entity type. */
  public static final EntityType OUT_TYPE = EntityType.get("efsm-graphml");

  /** The output GraphML-file-name command line parameter. */
  public static final Parameter EFSM_PARAM =
      new Parameter("efsm-graphml", true, "Output GraphML file");

  // The default file name.
  private static final String EFSM_GRAPHML_FILE = "efsm-model.graphml";

  // The printer result.
  private FileNames fileNames;

  /**
   * The default constructor.
   */
  public EfsmGraphMlPrinter() {
    super(EfsmGraphMlPrinter.ENGINE_ID, new EfsmGraphMlVisitor(), EfsmGraphMlPrinter.OUT_TYPE);
    addParameter(EFSM_PARAM);
  }

  @Override
  public void initialise(Map<EntityType, Entity> inputs) {
    final Arguments arguments = (Arguments)inputs.get(Arguments.TYPE);

    try {
      // Get arguments from command line.
      Arguments engineArgs = parseCommandLine(arguments.getCommandLine());

      // Create GraphML file.
      fileNames = new FileNames("graphml");
      fileNames.add(engineArgs.getValue(EFSM_PARAM, EFSM_GRAPHML_FILE));

      // Add graphML file to visitor.
      initGraphMlVisitor(fileNames.getList().get(0));
    } catch (ParseException e) {
      e.printStackTrace();
    }
  }
  
  /**
   * Initialize visitor with the graphML file name.
   * 
   * @param visitorName the output graphML file name.
   */
  public void initGraphMlVisitor(String visitorName)
  {
    ((EfsmGraphMlVisitor)this.visitor).initialize(visitorName);
  }
 
  @Override
  public Entity getOutput() { return this.fileNames; }
  
  /**
   * Print the specified {@link ru.ispras.retrascope.model.efsm.EfsmModel} model to GraphML file.
   * 
   * @param fileName the output file name.
   * @param efsm extended finite state machine model to be printed.
   */
  public void print(String fileName, EfsmModel efsm) {
    // TODO Delete

    // Create output file for GML printer
    EfsmGraphMlVisitor efsmGraphMlVisitor = new EfsmGraphMlVisitor();
    efsmGraphMlVisitor.initialize(fileName);
    
    // Construct cfg walker for GML printer
    EfsmModelWalker efsmWalker = new EfsmModelWalker(efsmGraphMlVisitor);
    // Run the cfg walker
    efsmWalker.start(efsm);
  }

}
