/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.model.basis;

import ru.ispras.fortress.util.InvariantChecks;

/**
 * Module variable data.
 * 
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class VariableData extends MetaInfo {

  /* The variable type. */
  private final VariableType variableType;

  /**
   * Constructs variable data from the specified variable type.
   *
   * @param variableType variable type
   *
   * @throws java.lang.NullPointerException when argument is {@code null}.
   */
  public VariableData(VariableType variableType) {
    InvariantChecks.checkNotNull(variableType);

    this.variableType = variableType;
  }

  /**
   * Returns variable type.
   *
   * @return variable type
   */
  public VariableType getVariableType() { return variableType; }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    }
    if (o == null || getClass() != o.getClass()) {
      return false;
    }

    VariableData that = (VariableData) o;

    return this.variableType == that.variableType;
  }

  @Override
  public int hashCode() { return this.variableType.hashCode(); }
}
