/*
 * Copyright 2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.model.basis;

/**
 * Variable types.
 * 
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public enum VariableType {
  IN,    // input signal
  OUT,   // output signal
  INOUT, // input-output signal
  REG;     // register, that is internal variable

  /**
   * Checks rather identifier is of signal type.
   *
   * @param id type identifier
   *
   * @return {@code true} if identifier is of input/output/input-output signal type,
   * {@code false} otherwise.
   */
  public static boolean isSignal (Enum<?> id) {

    return id.getClass().equals(VariableType.class)
      && (id == IN || id == OUT || id == INOUT);
  }

  /**
   * Checks rather identifier is of input or input-output signal type.
   *
   * @param id type identifier
   *
   * @return {@code true} if identifier is of input/input-output signal type,
   * {@code false} otherwise.
   */
  public static boolean isInputSignal(Enum<?> id) { return isSignal(id) && id != OUT; }
}
