/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
package ru.ispras.retrascope.model.cfg;

import ru.ispras.fortress.expression.NodeVariable;

import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.Set;

/**
 * Sink (final) node of control flow graph.
 *
 * Sink node does not have child nodes and only one parent node.
 * 
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class Sink extends CfgNode {

  /**
   * The default constructor.
   */
  public Sink() {}

  @Override
  public CfgModelNode deepcopy() { return new Sink(); }

  @Override
  public boolean hasParents() { return this.parentNode != null; }

  @Override
  public CfgModelNode getOnlyChild() { return null; }

  /**
   * Throws an exception.
   *
   * @param node child node
   *
   * @throws java.lang.UnsupportedOperationException every time it is called.
   */
  @Override
  public void addChild(CfgModelNode node) {
    throw new UnsupportedOperationException(
        String.format("No child nodes for: %s.", this.getType()));
  }

  /**
   * Throws an exception.
   *
   * @param node child node
   *
   * @throws java.lang.UnsupportedOperationException every time it is called.
   */
  @Override
  public void removeChild(CfgModelNode node) {
    throw new UnsupportedOperationException(
      String.format("No child nodes for: %s.", this.getType()));
  }

  @Override
  public Collection<CfgModelNode> getChildren() { return new LinkedHashSet<>(); }

  @Override
  public CfgNodeType getType() { return CfgNodeType.SINK; }

  @Override
  public boolean hasChildren() { return false; }

  @Override
  public CfgModelNode getOnlyParent() { return this.parentNode; }

  @Override
  public Set<NodeVariable> getUses() { return new LinkedHashSet<>(); }

  @Override
  public Set<NodeVariable> getDefines() { return new LinkedHashSet<>(); }

//  @Override
//  public boolean equals(Object o) {
//    return this == o || !(o == null || getClass() != o.getClass());
//  }
//
//  @Override
//  public int hashCode() { return 0; }
}
