/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ru.ispras.retrascope.model.cgaa;

import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.model.cfg.CfgModel;

/**
 * A clocked guarded atomic actions model.
 *
 * A collection of clocked guarded atomic actions
 * that are stored in the guarded actions decision diagram form
 * of control flow graph {@link ru.ispras.retrascope.model.cfg.CfgModel} model.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class CgaaModel extends Entity {

  /* The entity type. */
  public static final EntityType TYPE = EntityType.get("cgaa");

  /* The embedded control flow graph model. */
  private CfgModel model;

  /**
   * The default constructor.
   *
   * Constructs an entity from the specified
   * {@link ru.ispras.retrascope.model.cfg.CfgModel} control flow graph model.
   *
   * @param model the model.
   *
   * @throws java.lang.NullPointerException when argument is {@code null}.
   */
  public CgaaModel(CfgModel model) {
    super(CgaaModel.TYPE);
    InvariantChecks.checkNotNull(model);

    this.model = model;
  }

  /**
   * Adds the entity.
   *
   * @param entity the entity to be added.
   *
   * @throws java.lang.NullPointerException when argument is {@code null}.
   */
  @Override
  public void add(Entity entity) {
    InvariantChecks.checkNotNull(entity);

    if (entity instanceof CgaaModel) {

      this.model.add(((CgaaModel) entity).getModel());

    } else {
      throw new IllegalArgumentException(
          String.format("Incorrect type: %s.", entity.getEntityType().getId()));
    }
  }

  /**
   * Returns control flow graph model.
   *
   * @return model
   */
  public CfgModel getModel() { return this.model; }
}
