/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ru.ispras.retrascope.parser.vhdl;

import ru.ispras.fortress.expression.Node;
import ru.ispras.fortress.util.InvariantChecks;

/**
 * This object keeps results of some Zamia IG (Instantiation Graph) object
 * by {@link VhdlIgOperationParser} VHDL parser component.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
final class VhdlIgOperationParserResult {

  /* Expression to which IG object may be processed. */
  private final Node expr;

  /* Object to which IG object processing result may be stored
   * when it can not be transformed into expression. */
  private final Object data;

  /**
   * Constructs result object with the specified expression.
   *
   * @param expr expression
   *
   * @throws java.lang.NullPointerException when argument is {@code null}.
   */
  public VhdlIgOperationParserResult(Node expr) {
    InvariantChecks.checkNotNull(expr);

    this.expr = expr;
    this.data = null;
  }

  /**
   * Constructs result object with the specified internal object.
   *
   * @param obj internal object
   *
   * @throws java.lang.NullPointerException when argument is {@code null}.
   */
  public VhdlIgOperationParserResult(Object obj) {
    InvariantChecks.checkNotNull(obj);

    this.expr = null;
    this.data = obj;
  }
  /**
   * Returns expression that is stored in this object.
   *
   * @return expression
   */
  public Node getExpr() { return this.expr; }

  /**
   * Returns data that is stored in this object.
   *
   * @return data object
   */
  public Object getData() { return this.data; }
}
