/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.result.conflict;

import java.util.Collection;
import java.util.HashSet;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlSeeAlso;

import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.engine.efsm.extractor.conflict.EfsmConflictExtractor;

/**
 * Represents the result of {@link EfsmConflictExtractor} engine execution.
 * 
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.FIELD)
@XmlSeeAlso({Conflict.class})
public final class ConflictModel extends Entity {

  @XmlElementWrapper(name = "Conflicts")
  @XmlElement(name = "conflict")
  private Collection<Conflict> conflicts;

  public static final EntityType TYPE = EntityType.get("conflicts");

  /**
   * Constructs the ConflictModel class.
   */
  public ConflictModel() {
    super(TYPE);
    this.conflicts = new HashSet<>();
  }

  /**
   * Constructs the ConflictModel class using the specified set of conflicts.
   * 
   * @param conflicts the set of {@link Conflict}.
   */
  public ConflictModel(Collection<Conflict> conflicts) {
    super(TYPE);
    this.conflicts = new HashSet<>(conflicts);
  }

  @Override
  public void add(Entity entity) {
    if (entity instanceof ConflictModel) {
      this.addConflicts(((ConflictModel) entity).getConflicts());
    } else {
      throw new IllegalArgumentException(String.format("Unknown node kind: %s.", entity
          .getEntityType().getId()));
    }
  }

  /**
   * Adds a specified conflict to the set.
   * 
   * @param conflict the specified {@link Conflict}.
   */
  public void addConflict(Conflict conflict) {
    conflicts.add(conflict);
  }

  /**
   * Adds a specified collection of conflicts to the set.
   * 
   * @param conflicts the specified collection of {@link Conflict}.
   */
  public void addConflicts(Collection<Conflict> conflicts) {
    this.conflicts.addAll(conflicts);
  }

  /**
   * Returns the set of conflicts.
   * 
   * @return The {@link Collection} of conflicts.
   */
  public Collection<Conflict> getConflicts() {
    return conflicts;
  }

}
