library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use std.textio.all;
use IEEE.std_logic_textio.all;

package decider_parser is
	procedure extract_next_value(s: inout string; value_string: out string);

	procedure string_to_std_logic(s: inout string; value: out std_logic);
	procedure string_to_std_logic_vector(s: inout string; value: out std_logic_vector);
	procedure string_to_bit(s: inout string; value: out bit);
	procedure string_to_integer(s: inout string; value: out integer);
	procedure string_to_real(s: inout string; value: out real);
end decider_parser;

package body decider_parser is
	procedure extract_next_value(s: inout string; value_string: out string) is
		variable i, j: integer;
	begin
		i := 1;
		j := 1;
		value_string := (others => '/');

		while (s(i) /= ' ') loop
			if (s(i) /= '/') then
				value_string(j) := s(i);
				j := j + 1;
			end if;

			i := i + 1;
		end loop;

		s(i) := '/';
	end extract_next_value;

	procedure string_to_std_logic(s: inout string; value: out std_logic) is
		variable value_line: line;
		variable value_string: string (1 to %MAX_VALUE_LENGTH%);
	begin
			extract_next_value(s, value_string);
			write(value_line, value_string);
			read(value_line, value);
	end string_to_std_logic;

	procedure string_to_std_logic_vector(s: inout string; value: out std_logic_vector) is
		variable value_line: line;
		variable value_string: string (1 to %MAX_VALUE_LENGTH%);
	begin
			extract_next_value(s, value_string);
			write(value_line, value_string);
			read(value_line, value);
	end string_to_std_logic_vector;

	procedure string_to_bit(s: inout string; value: out bit) is
		variable value_line: line;
		variable value_string: string (1 to %MAX_VALUE_LENGTH%);
	begin
			extract_next_value(s, value_string);
			write(value_line, value_string);
			read(value_line, value);
	end string_to_bit;

	procedure string_to_integer(s: inout string; value: out integer) is
		variable value_line: line;
		variable value_string: string (1 to %MAX_VALUE_LENGTH%);
	begin
			extract_next_value(s, value_string);
			write(value_line, value_string);
			read(value_line, value);
	end string_to_integer;

	procedure string_to_real(s: inout string; value: out real) is
		variable value_line: line;
		variable value_string: string (1 to %MAX_VALUE_LENGTH%);
	begin
			extract_next_value(s, value_string);
			write(value_line, value_string);
			read(value_line, value);
	end string_to_real;
end decider_parser;