/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.ast.printer.smv;


import java.util.Map;

import org.apache.commons.cli.ParseException;

import ru.ispras.retrascope.basis.Arguments;
import ru.ispras.retrascope.basis.Engine;
import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.basis.FileNames;
import ru.ispras.retrascope.basis.Parameter;
import ru.ispras.retrascope.result.ast.AstModel;
import ru.ispras.retrascope.result.ast.AstVisitor;
import ru.ispras.retrascope.result.ast.AstWalker;

/**
 * This engine prints an {@link AstModel} to a file in SMV format.
 * 
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public final class AstSmvPrinter extends Engine {

  /* The engine identifier. */
  public static final String ENGINE_ID = "ast-smv-printer";
  
  /* The engine output entity type. */
  public static final EntityType OUT_TYPE = EntityType.get("ast-smv");

  /* The command-line parameter. */
  public static final Parameter SMV_FILE_NAME = new Parameter("output-file", true,
      "Output file name");

  /**
   * Constructs a new printer object.
   */
  public AstSmvPrinter() {
    super(ENGINE_ID);

    addInputType(AstModel.TYPE);
    setOutputType(OUT_TYPE);

    addParameter(SMV_FILE_NAME);

  }

  @Override
  public Entity start(Map<EntityType, Entity> inputs) {
    FileNames result = new FileNames("smv");
    final AstModel astModel = (AstModel) inputs.get(AstModel.TYPE);

    try {
      final Arguments arguments =
          this.parseCommandLine(((Arguments) inputs.get(Arguments.TYPE)).getCommandLine());
      final String fileName = arguments.getValue(SMV_FILE_NAME, "ast.smv");
      result.add(fileName);
      final AstVisitor visitor = new AstSmvVisitor(fileName);
      final AstWalker walker = new AstWalker(visitor);

      walker.start(astModel);

    } catch (ParseException e) {
      e.printStackTrace();
    }

    return result;
  }

}
