/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.cfg.transformer.ast;

import java.util.Collection;
import java.util.Map;

import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.engine.cfg.CfgEngine;
import ru.ispras.retrascope.model.cfg.CfgModel;
import ru.ispras.retrascope.model.cfg.CfgVisitor;
import ru.ispras.retrascope.model.cfg.Module;
import ru.ispras.retrascope.result.ast.Ast;
import ru.ispras.retrascope.result.ast.AstModel;

/**
 * This engine is used to transform Cfg entity to Ast entity.
 * 
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public final class CfgAstTransformer extends CfgEngine {

  public static final String ENGINE_ID = "cfg-ast-transformer";
  private CfgVisitor visitor;

  /**
   * Constructs a new transformer object.
   */
  public CfgAstTransformer() {
    super(ENGINE_ID, CfgModel.TYPE, AstModel.TYPE, new CfgAstVisitor());
  }

  @Override
  public void initialize(Map<EntityType, Entity> inputs) {
    visitor = this.getInitVisitor();
    ((CfgAstVisitor) visitor).initialize();
  }

  @Override
  public Entity getOutput() {
    Map<Module, Collection<Ast>> result = ((CfgAstVisitor) visitor).getAst();

    return new AstModel(result);
  }

}
