/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
package ru.ispras.retrascope.engine.testbench;

import java.io.IOException;
import java.nio.file.Paths;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.retrascope.basis.Engine;
import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.basis.VerilogTestbench;
import ru.ispras.retrascope.result.cfginterface.CfgInterface;
import ru.ispras.retrascope.result.cfginterface.CfgSignalInterface;
import ru.ispras.retrascope.result.testbench.Test;

/**
 * 
 * 
 * @author <a href="mailto:i.melnichenko@deltasolutions.ru">Igor Melnichenko</a>
 */
public class TestVerilogTestbenchPrinter extends Engine {
  public static final String ENGINE_ID = "test-verilog-testbench-printer".intern();
  public static final String TEST_FILE_EXTENSION = ".tst".intern();

  public TestVerilogTestbenchPrinter() {
    super(TestVerilogTestbenchPrinter.ENGINE_ID, EntityType.get(Test.ENTITY_TYPE_ID),
        EntityType.get(VerilogTestbench.ENTITY_TYPE_ID));
    this.addInputType(CfgInterface.TYPE);
  }

  @Override
  public Entity start(Map<EntityType, Entity> inputs) {
    Test test = (Test) inputs.get(EntityType.get(Test.ENTITY_TYPE_ID));
    CfgInterface cfgInterface = (CfgInterface) inputs.get(CfgInterface.TYPE);

    if (test == null) {
      throw new RuntimeException("No test entity has been found among inputs");
    }

    if (cfgInterface == null) {
      throw new RuntimeException("No interface entity has been found among inputs");
    }

    Map<String, CfgSignalInterface> interfaceMapping = cfgInterface.getMapping();

    if (cfgInterface.getMapping().isEmpty()) {
      throw new RuntimeException("No interface information has been provided");
    }

    if (cfgInterface.getMapping().size() > 1) {
      throw new RuntimeException("CfgInterface containing multiple modules is unsupported");
    }

    String moduleName = interfaceMapping.keySet().iterator().next();
    CfgSignalInterface externalSignals = interfaceMapping.get(moduleName);
    List<NodeVariable> inputSignals = externalSignals.getInputs();
    List<NodeVariable> outputSignals = externalSignals.getOutputs();
    Set<String> inputSignalNames = new HashSet<>(externalSignals.getInputs().size());

    for (NodeVariable inputSignal: inputSignals) {
      inputSignalNames.add(inputSignal.getName());
    }

    try {
      DeciderLikeTestPatternPrinter.print(test,
          Paths.get(moduleName + TestVerilogTestbenchPrinter.TEST_FILE_EXTENSION), inputSignalNames);
    } catch (IOException e) {
      throw new RuntimeException(e);
    }

    return null;
  }

}
