/*
 * Copyright 2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.model.basis.codeloc;

import ru.ispras.fortress.util.InvariantChecks;

import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.Set;

/**
 * Container for lines of code.
 *
 * Lines of code are stored in string format.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class CodeLineContainer {
  private final Set<String> container; // code lines container

  /**
   * Default constructor
   */
  public CodeLineContainer() {
    container = new LinkedHashSet<>();
  }

  /**
   * Adds line.
   *
   * @param line line of code
   */
  public void add(String line) {
    InvariantChecks.checkNotNull(line);

    container.add(line);
  }

  /**
   * Removes line.
   *
   * @param line line of code
   */
  public void remove(String line) {
    InvariantChecks.checkNotNull(line);

    container.remove(line);
  }

  /**
   * Returns all code lines.
   *
   * @return code line collection
   */
  public Collection<String> getLines() {
    return this.container;
  }
}
