/*
 * Copyright 2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.model.basis.codeloc;

import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.retrascope.model.basis.Range;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

/**
 * Mapping of code line ranges and code lines.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class CodeLineMap {

  private Map<Range, CodeLineContainer> map; // number-line mapping

  /**
   * Default constructor
   */
  public CodeLineMap() { this.map = new HashMap<>(); }

  /**
   * Returns mapping of line numbers into code lines.
   *
   * @return mapping of code line number ranges and code lines
   */
  public Map<Range, CodeLineContainer> getMap() { return map; }

  /**
   * Sets mapping of line numbers into code lines.
   *
   * @param map mapping of code line number ranges and code lines
   */
  public void setMap(Map<Range, CodeLineContainer> map) {
    InvariantChecks.checkNotNull(map);

    this.map = map;
  }

  /**
   * Returns all code lines.
   *
   * @return code lines collection
   */
  public Collection<CodeLineContainer> getAllCodeLines() {
    return this.map.values();
  }
}
