/*
 * Copyright 2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.model.basis.codeloc;

import ru.ispras.fortress.util.InvariantChecks;

/**
 * Location of some part of source code.
 *
 * It contains full name of file where it is situated and numbers of corresponding code lines.
 * 
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class CodeLocation {
  private FullFileName fullFileName; // full file name
  private CodeLineMap codeLineMap; // code lines mapping

  /**
   * Default constructor
   */
  public CodeLocation() {
    this.fullFileName = null;
    this.codeLineMap = new CodeLineMap();
  }

  /**
   * Returns full file name.
   *
   * @return full file name
   */
  public FullFileName getFullFileName() {
    return fullFileName;
  }

  /**
   * Sets full file name.
   *
   * @param name full file name
   */
  public void setFullFileName(FullFileName name) {
    InvariantChecks.checkNotNull(name);

    this.fullFileName = name;
  }

  /**
   * Returns code line mapping.
   *
   * @return code line mapping
   */
  public CodeLineMap getCodeLineMap() {
    return codeLineMap;
  }

  /**
   * Sets code line mapping.
   *
   * @param map code line mapping
   */
  public void setCodeLineMap(CodeLineMap map) {
    if (map == null)
      throw new NullPointerException();

    this.codeLineMap = map;
  }
}
