/*
 * Copyright 2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.model.basis.memory;

import ru.ispras.fortress.util.InvariantChecks;

/**
 * Memory model cell.
 * 
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 * 
 */
public class Bit {

  private final int num; // bit number at memory
  private BitValue value; // bit value

  /**
   * Default constructor
   *
   * @param num bit number
   */
  public Bit(int num) { this.num = num; }

  /**
   * Returns bit number.
   *
   * @return bit number
   */
  public int getNum() { return num; }

  /**
   * Returns bit value.
   *
   * @return bit value
   */
  public BitValue getValue() { return value; }

  /**
   * Sets bit value.
   *
   * @param value bit value
   */
  public void setValue(BitValue value) {
    InvariantChecks.checkNotNull(value);

    this.value = value;
  }
}
