/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.result.ast;

import java.util.Collection;
import java.util.LinkedHashSet;

import ru.ispras.retrascope.model.cfg.Process;

/**
 * This class represents the abstract syntax tree based on the Cfg {@link Process}.
 * 
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public class Ast {

  private Collection<AstNode> nodes;
  private AstNode rootNode;
  private Process parentProcess;

  /**
   * Constructs a new abstract syntax tree.
   */
  public Ast() {
    nodes = new LinkedHashSet<AstNode>();
    rootNode = null;
  }

  /**
   * Constructs a new abstract syntax tree using the specified collection of nodes.
   * 
   * @param astNodes - the specified collection of {@link AstNode}.
   */
  public Ast(Collection<AstNode> astNodes) {
    nodes = astNodes;
    rootNode = null;
  }

  /**
   * Returns the collection of tree nodes.
   * 
   * @return The collection of {@link AstNode}.
   */
  public Collection<AstNode> getNodes() {
    return this.nodes;
  }

  /**
   * Adds a node to a tree.
   * 
   * @param node - The specified {@link AstNode}.
   */
  public void addNode(AstNode node) {
    nodes.add(node);
  }

  /**
   * Sets the root node of a tree.
   * 
   * @param node - The specified {@link AstNode}.
   */
  public void setRootNode(AstNode node) {
    node.setRoot(true);
    this.rootNode = node;
  }

  /**
   * Returns the root node of a tree.
   * 
   * @return The root {@link AstNode}.
   */
  public AstNode getRootNode() {
    return this.rootNode;
  }

  /**
   * Sets the process associated with this tree.
   * 
   * @param process - the specified {@link Process}.
   */
  public void setParentProcess(Process process) {
    this.parentProcess = process;
  }

  /**
   * Returns the process associated with this tree.
   * 
   * @return The associated {@link Process}.
   */
  public Process getParentProcess() {
    return this.parentProcess;
  }

}
