/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.result.ast;

import ru.ispras.retrascope.model.cfg.BasicBlock;
import ru.ispras.retrascope.model.cfg.Case;
import ru.ispras.retrascope.model.cfg.Module;
import ru.ispras.retrascope.model.cfg.Switch;

/**
 * Visitor interface for {@link AstModel}.
 * 
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public interface AstVisitor {

  /**
   * Starts visiting an {@link AstModel}.
   * 
   * @param model - the specified model.
   */
  void onModelBegin(AstModel model);

  /**
   * Finishes visiting an {@link AstModel}.
   * 
   * @param model - the specified model.
   */
  void onModelEnd(AstModel model);

  /**
   * Starts visiting a module.
   * 
   * @param module - the specified module.
   */
  void onModuleBegin(Module module);

  /**
   * Finishes visiting a module.
   * 
   * @param module - the specified module.
   */
  void onModuleEnd(Module module);

  /**
   * Starts visiting an abstract syntax tree.
   * 
   * @param tree - the specified Ast.
   */
  void onTreeBegin(Ast tree);

  /**
   * Finishes visiting an abstract syntax tree.
   * 
   * @param tree - the specified Ast.
   */
  void onTreeEnd(Ast tree);

  /**
   * Starts visiting a root node.
   */
  void onRootNodeBegin();

  /**
   * Finishes visiting a root node.
   */
  void onRootNodeEnd();

  /**
   * Starts visiting a node.
   * 
   * @param node - the specified {@link AstNode}.
   */
  void onNodeBegin(AstNode node);

  /**
   * Finishes visiting a node.
   * 
   * @param node - the specified {@link AstNode}.
   */
  void onNodeEnd(AstNode node);

  /**
   * Starts visiting a node associated with a {@link BasicBlock}.
   * 
   * @param node - the specified {@link AstNode}
   */
  void onBasicBlockBegin(AstNode node);

  /**
   * Finishes visiting a node associated with a {@link BasicBlock}.
   * 
   * @param node - the specified {@link AstNode}
   */
  void onBasicBlockEnd(AstNode node);

  /**
   * Starts visiting a node associated with a {@link Switch}.
   * 
   * @param node - the specified {@link AstNode}
   */
  void onSwitchBegin(AstNode node);

  /**
   * Finishes visiting a node associated with a {@link Switch}.
   * 
   * @param node - the specified {@link AstNode}
   */
  void onSwitchEnd(AstNode node);

  /**
   * Starts visiting a node associated with a {@link Case}.
   * 
   * @param node - the specified {@link AstNode}
   */
  void onCaseBegin(AstNode node);

  /**
   * Finishes visiting a node associated with a {@link Case}.
   * 
   * @param node - the specified {@link AstNode}
   */
  void onCaseEnd(AstNode node);

}
