/*
 * Copyright 2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.testbench;

import java.util.HashMap;

import ru.ispras.retrascope.testbench.message.MessageField;

/**
 * The class for storing mappings between hardware device signals and corresponding model message
 * fields.
 * 
 */
public class Adapter implements Xmlable {
  private final HashMap<MessageField, Signal> fieldsToSignalsMapping;
  private final HashMap<Signal, MessageField> signalsToFieldsMapping;

  /**
   * Constructs the empty adapter.
   */
  public Adapter() {
    this.fieldsToSignalsMapping = new HashMap<MessageField, Signal>();
    this.signalsToFieldsMapping = new HashMap<Signal, MessageField>();
  }

  /**
   * Adds to the adapter a mapping between specified signal and message field.
   * 
   * @param signal - a signal for mapping
   * @param field - a message field for mapping
   */
  public void addMapping(Signal signal, MessageField field) {
    this.fieldsToSignalsMapping.put(field, signal);
    this.signalsToFieldsMapping.put(signal, field);
  }

  /**
   * Returns a signal which the specified field mapped to.
   * 
   * @return a signal which the specified message field is mapped to
   */
  public Signal getMapping(MessageField field) {
    return this.fieldsToSignalsMapping.get(field);
  }

  /**
   * Returns a message field which the specified field mapped to.
   * 
   * @return a message field which the specified signal is mapped to
   */
  public MessageField getMapping(Signal signal) {
    return this.signalsToFieldsMapping.get(signal);
  }

  /**
   * Returns an XML representation of the adapter.
   * 
   * @return an XML representation of the adapter.
   */
  @Override
  public String toXML() {
    StringBuilder xml = new StringBuilder();

    xml.append("<adapter>");
    xml.append(Adapter.lineSeparator);
    for (MessageField field : this.fieldsToSignalsMapping.keySet()) {
      xml.append("<mapping fieldName=\"" + field.getName() + "\" signalName=\""
          + this.fieldsToSignalsMapping.get(field).getName() + "\" />");
    }
    xml.append(Adapter.lineSeparator);
    xml.append("</adapter>");

    return xml.toString();
  }
}
