/*
 * Copyright 2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.testbench;

/**
 * @author <a href="mailto:razdobarin@ispras.ru">Dmitry Razdobarin</a>
 */
public class Signal implements Xmlable {
  private final String name;
  private final int capacity;

  /**
   * Constructs the signal with the specified name and capacity.
   * 
   * @param signalName - a name.
   * @param signalCapacity - a capacity.
   */
  public Signal(String signalName, int signalCapacity) {
    this.name = signalName;
    this.capacity = signalCapacity;
  }

  /**
   * Returns a name of the signal.
   * 
   * @return a name of the signal.
   */
  public String getName() {
    return this.name;
  }

  /**
   * Returns a capacity of the signal.
   * 
   * @return a capacity of the signal.
   */
  public int getCapacity() {
    return this.capacity;
  }

  /**
   * Returns an XML representation of the signal.
   * 
   * @return an XML representation of the signal.
   */
  @Override
  public String toXML() {

    return ("<signal name = \"" + name +
            "\" capacity = \"" + capacity + "\"/>") + Signal.lineSeparator;
  }
}
