/*
 * Copyright 2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.testbench.message;

import java.util.ArrayList;
import java.util.List;

import ru.ispras.fortress.solver.constraint.Constraint;
import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.retrascope.testbench.Xmlable;

/**
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class ConstraintTree implements Xmlable {
  private final Constraint constraint;
  private List<ConstraintTree> nodes;
  private final ENodeType type;

  public enum ENodeType { OR_NODE, AND_NODE, LEAF_NODE }

  public ConstraintTree(Constraint constraint, ENodeType type) {
    if ((type != ENodeType.LEAF_NODE) && (constraint != null))
      throw new IllegalStateException("Non-leaf node can not have not null constraint");
    this.constraint = constraint;
    this.type = type;
    if (type != ENodeType.LEAF_NODE)
      this.nodes = new ArrayList<>();
  }

  public boolean addNode(ConstraintTree node) {
    InvariantChecks.checkNotNull(node);

    if (this.type == ENodeType.LEAF_NODE)
      throw new IllegalStateException("Node cannot be added to tree leaf");
    else
      return this.nodes.add(node);
  }

  public void addLeaf(Constraint constraint) {
    ConstraintTree leafTree = new ConstraintTree(constraint, ENodeType.LEAF_NODE);
    this.addNode(leafTree);
  }

  @Override
  public String toXML() {
    // TODO: implement when JCS API will be ready
    return null;
  }

  public Constraint getConstraint() { return this.constraint; }

  public ENodeType getType() { return this.type; }
}
