/*
 * Copyright 2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.testbench.message;

import java.util.HashMap;
import java.util.Map;

import ru.ispras.fortress.data.Data;
import ru.ispras.fortress.data.DataType;

/**
 * @author <a href="mailto:razdobarin@ispras.ru">Dmitry Razdobarin</a>
 */
public class Message {
  private final MessageType type;
  private final HashMap<MessageField, Data> fieldValues;

  public Message(MessageType messageType) {
    this.type = messageType;
    this.fieldValues = new HashMap<>(type.getFields().size());
    for (MessageField field : messageType.getFields().values())
      this.fieldValues.put(field, DataType.BIT_VECTOR(field.getSize()).valueOf("0", 2));
  }

  public void setFieldValue(MessageField field, Data value) {
    this.fieldValues.put(field, value);
  }

  public MessageType getType() {
    return this.type;
  }

  public Map<MessageField, Data> getFieldValues() {
    return this.fieldValues;
  }
}
