/*
 * Copyright 2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.testbench.message;

import ru.ispras.retrascope.testbench.Xmlable;

/**
 * @author <a href="mailto:razdobarin@ispras.ru">Dmitry Razdobarin</a>,
 */
public class MessageField implements Xmlable {
  private final String name;
  private final int size;
  private final Integer mask;
  private boolean incomparable;

  public MessageField(String name, int capacity, Integer mask) {
    this.name = name;
    this.size = capacity;
    this.mask = mask;
    this.incomparable = false;
  }

  public MessageField(String name, int capacity, Integer mask, boolean incomparable) {
    this.name = name;
    this.size = capacity;
    this.mask = mask;
    this.setIncomparable(incomparable);
  }

  public String getName() {
    return this.name;
  }

  public int getSize() {
    return this.size;
  }

  public int getMask() {
    return this.mask;
  }

  public boolean isIncomparable() {
    return this.incomparable;
  }

  public void setIncomparable(boolean incomparable) {
    this.incomparable = incomparable;
  }

  @Override
  public String toXML() {

    return ("<messageField name=\"" + name + "\" capacity=\"" + this.size + "\" mask=\"0x"
      + Integer.toHexString(mask).toUpperCase() + "\" incomparable=\"" + this.incomparable
      + "\"/>");
  }
}
