/*
 * Copyright 2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.testbench.message;

import java.util.HashMap;

import ru.ispras.retrascope.testbench.Xmlable;

/**
 * @author <a href="mailto:razdobarin@ispras.ru">Dmitry Razdobarin</a>,
 */
public class MessageType implements Xmlable {
  private String name;
  private HashMap<String, MessageField> fields;
  private ConstraintTree tree;

  public MessageType(String name) {
    this.setName(name);
  }

  public MessageType(String name, ConstraintTree tree) {
    this.name = name;
    this.tree = tree;
  }

  public void addField(MessageField field) {
    this.fields.put(field.getName(), field);
  }

  public void removeField(MessageField field) {
    this.fields.remove(field.getName());
  }

  /**
   * Returns the name of the message type represented by this MessageType object
   * 
   * @return {@link String} - name of the message type.
   */
  public String getName() {
    return name;
  }

  /**
   * Returns the name of the message type represented by this MessageType object
   * 
   * @return {@link HashMap} - a map containing message fields. Key is a field name, value is a
   *         corresponding MessageField object.
   */
  public HashMap<String, MessageField> getFields() {
    return fields;
  }

  public void setName(String name) {
    this.name = name;
  }

  @Override
  public String toXML() {
    /*
     * StringBuilder result = new StringBuilder();
     * 
     * result.append("<messagetype name = \"");
     * 
     * result.append("</messagetype>");
     * 
     * return result.toString();
     */
    return "";
  }

  /**
   * Returns the tree of constraints representing a part of a protocol related to this message type.
   * 
   * @return {@link ConstraintTree} - tree of constraints.
   */
  public ConstraintTree getTree() {
    return tree;
  }

  public void setTree(ConstraintTree tree) {
    this.tree = tree;
  }
}
