/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope;

import java.nio.file.Paths;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * A collection of VHDL files (grouped in projects)
 * that are used for testing.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class VhdlFiles {

  // VHDL projects & files and their top level modules names
  private static Map<String, Map<String,String>> map = new LinkedHashMap<>();

  static {

    //TODO: uncomment 'count' project when functions processing will be implemented
    /* The count project. */
//    Map<String, String> count = new HashMap<>();
//    count.put(Paths.get("src", "test", "vhdl", "count", "count.vhd").toString(),"count");
//    map.put("count", count);

    //TODO: uncomment all when necessary syntax constructions will be supported

    /* The Dlx project. */
//    Map<String, String> dlx = new HashMap<>();
//    dlx.put(Paths.get("src","test","vhdl","dlx","Dlx.vhd").toString(), "dlx");
//    dlx.put(Paths.get("src","test","vhdl","dlx","Environment.vhd").toString(), "environment");
//    map.put("dlx", dlx);

    /* The example project. */
//    Map<String, String> example = new LinkedHashMap<>();
//    example.put(Paths.get("src","test","vhdl","example","test.vhd").toString(), "test");
//    example.put(Paths.get("src","test","vhdl","example","driver.vhd").toString(), "driver");
//    example.put(Paths.get("src","test","vhdl","example","example.vhd").toString(), "example");
//    map.put("example", example);

    /* The HC11 project. */
//    Map<String, String> hc11 = new HashMap<>();
//    hc11.put(Paths.get("src","test","vhdl","hc11","clkgen.vhd").toString(), "clock");
//    hc11.put(Paths.get("src","test","vhdl","hc11","dev.vhd").toString(), "dev");
//    hc11.put(Paths.get("src","test","vhdl","hc11","hc11rtl.vhd").toString(), "hc11cpu");
//    hc11.put(Paths.get("src","test","vhdl","hc11","ram.vhd").toString(), "hc11ram");
//    hc11.put(Paths.get("src","test","vhdl","hc11","srec_rom.vhd").toString(), "srec_rom");
//    hc11.put(Paths.get("src","test","vhdl","hc11","syncore.vhd").toString(), "hc11core");
//    map.put("hc11", hc11);

    /* The  ITC'99  benchmark project. */
    Map<String, String> itc99 = new LinkedHashMap<>();
    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b01","b01.vhd").toString(), "b01");
    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b02","b02.vhd").toString(), "b02");
    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b03","b03.vhd").toString(), "b03");
    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b04","b04.vhd").toString(), "b04");
//    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b05","b05.vhd").toString(), "b05"); // abnormally many transitions
    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b06","b06.vhd").toString(), "b06");
    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b07","b07.vhd").toString(), "b07");
    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b08","b08.vhd").toString(), "b08");
    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b09","b09.vhd").toString(), "b09");
    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b10","b10.vhd").toString(), "b10");
    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b11","b11.vhd").toString(), "b11");
//    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b12","b12.vhd").toString(), "b12"); // loop
    //itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b13","b13.vhd").toString(), "b13");//shared variables
//    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b14","b14.vhd").toString(), "b14"); // too many diagram paths
//    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b14_1","b14_1.vhd").toString(), "b14_1"); // too many diagram paths
//    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b15","b15.vhd").toString(), "b15"); // too many diagram paths
//    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b15_1","b15_1.vhd").toString(), "b15_1"); // too many diagram paths
//    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b17","b17.vhd").toString(), "b17");
//    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b17_1","b17_1.vhd").toString(), "b17_1");
//    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b18","b18.vhd").toString(), "b18");
//    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b18_1","b18_1.vhd").toString(), "b18_1");
//    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b19","b19.vhd").toString(), "b19");
//    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b19_1","b19_1.vhd").toString(), "b19_1");
//    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b20","b20.vhd").toString(), "b20");
//    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b20_1","b20_1.vhd").toString(), "b20_1");
//    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b21","b21.vhd").toString(), "b21");
//    itc99.put(Paths.get("src","test","vhdl","itc99-poli2/b21_1","b21_1.vhd").toString(), "b21_1");
//    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b22","b22.vhd").toString(), "b22");
//    itc99.put(Paths.get("src","test","vhdl","itc99-poli2","b22_1","b22_1.vhd").toString(), "b22_1");
    map.put("itc99", itc99);

    /* The MiniMIPS project. */
//    Map<String, String> minimips = new HashMap<>();
//    minimips.put(Paths.get("src","test","vhdl","minimips","alu.vhd").toString(), "alu");
//    minimips.put(Paths.get("src","test","vhdl","minimips","banc.vhd").toString(), "banc");
//    minimips.put(Paths.get("src","test","vhdl","minimips","bus_ctrl.vhd").toString(), "bus_ctrl");
//    minimips.put(Paths.get("src","test","vhdl","minimips","minimips.vhd").toString(), "minimips");
//    minimips.put(Paths.get("src","test","vhdl","minimips","pps_di.vhd").toString(), "pps_di");
//    minimips.put(Paths.get("src","test","vhdl","minimips","pps_ei.vhd").toString(), "pps_ei");
//    minimips.put(Paths.get("src","test","vhdl","minimips","pps_ex.vhd").toString(), "pps_ex");
//    minimips.put(Paths.get("src","test","vhdl","minimips","pps_mem.vhd").toString(), "pps_mem");
//    minimips.put(Paths.get("src","test","vhdl","minimips","pps_pf.vhd").toString(), "pps_pf");
//    minimips.put(Paths.get("src","test","vhdl","minimips","ram.vhd").toString(), "ram");
//    minimips.put(Paths.get("src","test","vhdl","minimips","renvoi.vhd").toString(), "renvoi");
//    minimips.put(Paths.get("src","test","vhdl","minimips","rom.vhd").toString(), "rom");
//    minimips.put(Paths.get("src","test","vhdl","minimips","syscop.vhd").toString(), "syscop");
//    map.put("minimips", minimips);

    /* The Plasma project. */
//    Map<String, String> plasma = new HashMap<>();
//    plasma.put(Paths.get("src","test","vhdl","plasma","alu.vhd").toString(), "alu");
//    plasma.put(Paths.get("src","test","vhdl","plasma","bus_mux.vhd").toString(), "bus_mux");
//    plasma.put(Paths.get("src","test","vhdl","plasma","control.vhd").toString(), "control");
//    plasma.put(Paths.get("src","test","vhdl","plasma","mem_ctrl.vhd").toString(), "mem_ctrl");
//    plasma.put(Paths.get("src","test","vhdl","plasma","mlite2sram.vhd").toString(), "mlite2sram");
//    plasma.put(Paths.get("src","test","vhdl","plasma","mlite2uart.vhd").toString(), "mlite2uart");
//    plasma.put(Paths.get("src","test","vhdl","plasma","mlite_cpu.vhd").toString(), "mlite_cpu");
//    plasma.put(Paths.get("src","test","vhdl","plasma","mult.vhd").toString(), "mult");
//    plasma.put(Paths.get("src","test","vhdl","plasma","pc_next.vhd").toString(), "pc_next");
//    plasma.put(Paths.get("src","test","vhdl","plasma","pipeline.vhd").toString(), "pipeline");
//    plasma.put(Paths.get("src","test","vhdl","plasma","plasma.vhd").toString(), "plasma");
//    plasma.put(Paths.get("src","test","vhdl","plasma","ram.vhd").toString(), "ram");
//    plasma.put(Paths.get("src","test","vhdl","plasma","reg_bank.vhd").toString(), "reg_bank");
//    plasma.put(Paths.get("src","test","vhdl","plasma","shifter.vhd").toString(), "shifter");
//    plasma.put(Paths.get("src","test","vhdl","plasma","sram2mlite.vhd").toString(), "sram2mlite");
//    plasma.put(Paths.get("src","test","vhdl","plasma","uart.vhd").toString(), "uart");
//    map.put("plasma", plasma);

    /* The Uart project. */
//    Map<String, String> uart = new HashMap<>();
//    uart.put(Paths.get("src","test","vhdl","uart","slib_clock_div.vhd").toString(), "slib_clock_div");
//    uart.put(Paths.get("src","test","vhdl","uart","slib_counter.vhd").toString(), "slib_counter");
//    uart.put(Paths.get("src","test","vhdl","uart","slib_edge_detect.vhd").toString(), "slib_edge_detect");
//    uart.put(Paths.get("src","test","vhdl","uart","slib_fifo.vhd").toString(), "slib_fifo");
//    uart.put(Paths.get("src","test","vhdl","uart","slib_input_filter.vhd").toString(), "slib_input_filter");
//    uart.put(Paths.get("src","test","vhdl","uart","slib_input_sync.vhd").toString(), "slib_input_sync");
//    uart.put(Paths.get("src","test","vhdl","uart","slib_mv_filter.vhd").toString(), "slib_mv_filter");
//    uart.put(Paths.get("src","test","vhdl","uart","uart_16750.vhd").toString(), "uart_16750");
//    uart.put(Paths.get("src","test","vhdl","uart","uart_baudgen.vhd").toString(), "uart_baudgen");
//    uart.put(Paths.get("src","test","vhdl","uart","uart_interrupt.vhd").toString(), "uart_interrupt");
//    uart.put(Paths.get("src","test","vhdl","uart","uart_receiver.vhd").toString(), "uart_receiver");
//    uart.put(Paths.get("src","test","vhdl","uart","uart_transmitter.vhd").toString(), "uart_transmitter");
//    map.put("uart", uart);
  }

  public static Map<String, Map<String, String>> getFiles() { return map; }
}
