/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.engine.cfg.printer.graphml;

import org.junit.Test;
import ru.ispras.fortress.expression.printer.ExprPrinter;
import ru.ispras.retrascope.VerilogFiles;
import ru.ispras.retrascope.VhdlFiles;
import ru.ispras.retrascope.basis.Configuration;
import ru.ispras.retrascope.util.VhdlUtilTest;

import java.io.File;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * Module-level test for printer from control flow graph to GraphML format.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class CfgGraphMlPrinterModuleTestCase {

  @Test
  public void runTest() {

    /* Get VHDL projects. */
    Map<String, Map<String, String>> vhdlFiles = VhdlFiles.getFiles();

    //* Get Verilog projects. */
    Map<String, Map<String, String>> verilogFiles = VerilogFiles.getFiles();

    /* Printing in all the defined styles. */

    ExprPrinter[] printers = ExprPrinter.values();

    for (ExprPrinter printer : printers) {

      /* Run tool on each module of each VHDL project. */
      runToolOnFiles(vhdlFiles, printer);

      /* run tool on each module of each Verilog project. */
      runToolOnFiles(verilogFiles, printer);
    }
  }

  private void runToolOnFiles(Map<String, Map<String, String>> hdlFiles, ExprPrinter printer) {
    for (String project : hdlFiles.keySet()) {

      /* Get project container. */
      Map<String, String> projectMap = hdlFiles.get(project);

      for (String fileName : projectMap.keySet()) {

        /* Only-one-file-containing mapping. */
        Map<String, String> onlyFile = new LinkedHashMap<>();
        onlyFile.put(fileName, projectMap.get(fileName));

        /* Use design name for output GraphML file. */
        String graphMlFileName = fileName.substring(
            fileName.lastIndexOf(File.separatorChar) + 1,
            fileName.lastIndexOf('.'));

        /* Save the result on the project directory. */
        VhdlUtilTest.runVhdl(
            onlyFile, CfgGraphMlPrinter.OUT_TYPE.getId(),
            new String[]{
                "--" + Configuration.PRINT_STYLE_PARAM.getLongName(),
                printer.name(),
                "--" + CfgGraphMlPrinter.GRAPHML_PARAM.getLongName(),
                graphMlFileName + "-" + printer.name() + "-cfg.graphml"});
      }
    }
  }
}
