/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.engine.cgaa.printer.graphml;

import java.io.File;
import java.util.LinkedHashMap;
import java.util.Map;

import org.junit.Test;

import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.retrascope.VerilogFiles;
import ru.ispras.retrascope.VhdlFiles;
import ru.ispras.retrascope.engine.cfg.printer.graphml.CfgGraphMlPrinter;
import ru.ispras.retrascope.util.VerilogUtilTest;
import ru.ispras.retrascope.util.VhdlUtilTest;

/**
 * The test for GraphML printer {@link ru.ispras.retrascope.engine.cgaa.printer.graphml.CgaaGraphMlPrinter}
 * of clocked guarded atomic actions models {@link ru.ispras.retrascope.model.cgaa.CgaaModel}.
 *
 * Test runs the printer on every top-level module
 * of every HDL file from project test suite.

 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class CgaaGraphMlPrinterTestCase {
  @Test
  public void runTest() {

    /* Get VHDL projects. */
    Map<String, Map<String, String>> vhdlFiles = VhdlFiles.getFiles();

    /* Run the tool on each module of each VHDL project. */
    for (String map : vhdlFiles.keySet()) {
      Map<String, String> projectMap = vhdlFiles.get(map);

      for (String fileName : projectMap.keySet()) {
        Map<String, String> onlyFile = new LinkedHashMap<>();
        onlyFile.put(fileName, projectMap.get(fileName));
        VhdlUtilTest.runVhdl(
            onlyFile,
            CgaaGraphMlPrinter.OUT_TYPE.getId(),
            new String[] {
                "--" + CfgGraphMlPrinter.GRAPHML_PARAM.getLongName(),
                onlyFile.get(fileName) + "-cgaa.graphml"});
      }
    }

    /* Get Verilog projects. */
    Map<String, Map<String, String>> verilogFiles = VerilogFiles.getFiles();

    /* Run the tool on each module of each Verilog project. */
    for (String map : verilogFiles.keySet()) {
      Map<String, String> projectMap = verilogFiles.get(map);

      for (String fileName : projectMap.keySet()) {
        Map<String, String> onlyFile = new LinkedHashMap<>();
        onlyFile.put(fileName, projectMap.get(fileName));

        InvariantChecks.checkNotNull(onlyFile.get(fileName));

        /** For Verilog modules there are no toplevels,
        * so GraphML file name must be extracted from filename itself.
        */
        String graphMlFileName = fileName.substring(
            fileName.lastIndexOf(File.separatorChar) + 1,
            fileName.lastIndexOf('.'));
        VerilogUtilTest.runVerilog(
            onlyFile,
            CgaaGraphMlPrinter.OUT_TYPE.getId(),
            new String[] {
                "--" + CfgGraphMlPrinter.GRAPHML_PARAM.getLongName(),
                graphMlFileName + "-cgaa.graphml"});
      }
    }
  }
}
