/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ru.ispras.retrascope.engine.efsm.printer.graphml;

import org.junit.Test;
import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.retrascope.Retrascope;
import ru.ispras.retrascope.VerilogFiles;
import ru.ispras.retrascope.VhdlFiles;
import ru.ispras.retrascope.basis.Configuration;
import ru.ispras.retrascope.engine.cgaa.transformer.efsm.CgaaEfsmTransformer;
import ru.ispras.retrascope.util.VerilogUtilTest;
import ru.ispras.retrascope.util.VhdlUtilTest;

import java.io.File;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * Test for {@link ru.ispras.retrascope.engine.efsm.printer.graphml.EfsmGraphMlPrinter}
 * printer for {@link ru.ispras.retrascope.model.efsm.EfsmModel} extended finite
 * state machine model.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class EfsmGraphMlPrinterModuleTestCase {
  @Test
  public void runTest() {

    /* Get VHDL projects. */
    Map<String, Map<String, String>> vhdlFiles = VhdlFiles.getFiles();

    /* Run Retrascope on each module of each VHDL project. */
    for (String map : vhdlFiles.keySet()) {
      Map<String, String> projectMap = vhdlFiles.get(map);

      for (String fileName : projectMap.keySet()) {
        Map<String, String> onlyFile = new LinkedHashMap<>();
        onlyFile.put(fileName, projectMap.get(fileName));
        VhdlUtilTest.runVhdl(
          onlyFile,
          EfsmGraphMlPrinter.OUT_TYPE.getId(),
          new String[] {
              "--" + Configuration.PRINT_STYLE_PARAM.getLongName(),
              "VHDL",
              "--" + EfsmGraphMlPrinter.EFSM_PARAM.getLongName(),
              onlyFile.get(fileName) + "-efsm.graphml",
              "--" + Retrascope.ENGINE_PARAM.getLongName(),
              CgaaEfsmTransformer.ENGINE_ID});
      }
    }

    /* Get Verilog projects. */
    Map<String, Map<String, String>> verilogFiles = VerilogFiles.getFiles();

    /* Run Retrascope on each module of each Verilog project. */
    for (String map : verilogFiles.keySet()) {
      Map<String, String> projectMap = verilogFiles.get(map);

      for (String fileName : projectMap.keySet()) {
        Map<String, String> onlyFile = new LinkedHashMap<>();
        onlyFile.put(fileName, projectMap.get(fileName));

        InvariantChecks.checkNotNull(onlyFile.get(fileName));

        /** For Verilog modules there are no toplevels,
         * so GraphML file name must be extracted from filename itself. */
        String graphMlFileName = fileName.substring(
          fileName.lastIndexOf(File.separatorChar) + 1,
          fileName.lastIndexOf('.'));
        VerilogUtilTest.runVerilog(
          onlyFile,
          EfsmGraphMlPrinter.OUT_TYPE.getId(),
          new String[] {
              "--" + Configuration.PRINT_STYLE_PARAM.getLongName(), "VERILOG",
              "--" + EfsmGraphMlPrinter.EFSM_PARAM.getLongName(), graphMlFileName + "-efsm.graphml",
              "--" + Retrascope.ENGINE_PARAM.getLongName(), CgaaEfsmTransformer.ENGINE_ID});
      }
    }
  }
}
