/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
package ru.ispras.retrascope.engine.testbench;

import org.junit.After;
import org.junit.Test;
import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.basis.VhdlTestbench;
import ru.ispras.retrascope.engine.efsm.generator.test.AbstractEfsmTestGeneratorDummyTest;
import ru.ispras.retrascope.engine.efsm.generator.test.fate.EfsmFateTestGenerator;
import ru.ispras.retrascope.model.efsm.EfsmModel;
import ru.ispras.retrascope.model.efsm.examples.B04;
import ru.ispras.retrascope.model.efsm.examples.SimpleEfsm;
import ru.ispras.retrascope.model.efsm.examples.SimplePipelinedEfsm;
import ru.ispras.retrascope.result.cfginterface.CfgInterface;
import ru.ispras.retrascope.util.Log;
import ru.ispras.retrascope.util.LogLevel;

import java.io.BufferedReader;
import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import java.util.logging.Logger;

/**
 * 
 * 
 * @author <a href="mailto:i.melnichenko@deltasolutions.ru">Igor Melnichenko</a>
 */
public class TestVhdlTestbenchPrinterTestCase extends AbstractEfsmTestGeneratorDummyTest {
  private final Logger logger = Log.getLogger(this.getClass());
  private final String logHeader = "TestVhdlTestbenchPrinterTestCase: ";

  private final Set<String> fileNames = new HashSet<>();

  @Test
  public void generate() throws IOException {
    EfsmFateTestGenerator generator = new EfsmFateTestGenerator();

    for (EfsmModel efsmModel: this.efsmModels) {
      HashMap<EntityType, Entity> generatorInputs = new HashMap<>();
      generatorInputs.put(efsmModel.getEntityType(), efsmModel);

      ru.ispras.retrascope.result.testbench.Test test = generator.start(generatorInputs);
      CfgInterface cfgInterface = null;
      
      if (efsmModel.getEfsmList().contains(B04.EFSM)) {
        cfgInterface = B04.CFG_INTERFACE;
      } else {
        if (efsmModel.getEfsmList().contains(SimpleEfsm.EFSM)) {
          cfgInterface = SimpleEfsm.CFG_INTERFACE;
        } else {
          if (efsmModel.getEfsmList().contains(SimplePipelinedEfsm.EFSM)) {
            cfgInterface = SimplePipelinedEfsm.CFG_INTERFACE;
          }
        }
      }

      if (cfgInterface != null) {
        TestVhdlTestbenchPrinter printer = new TestVhdlTestbenchPrinter();

        HashMap<EntityType, Entity> testbenchPrinterInputs = new HashMap<>();
        testbenchPrinterInputs.
            put(EntityType.get(ru.ispras.retrascope.result.testbench.Test.ENTITY_TYPE_ID), test);
        testbenchPrinterInputs.put(CfgInterface.TYPE, cfgInterface);

       VhdlTestbench testbench = printer.start(testbenchPrinterInputs);

       for (String fileName: testbench.getArray()) {
         String fullFileName = fileName + testbench.getType();
         this.fileNames.add(fullFileName);

         this.logger.log(LogLevel.INFO,
             this.logHeader + "reading the testbench file: " + fullFileName);

         try (BufferedReader reader =
             Files.newBufferedReader(Paths.get(fullFileName), Charset.defaultCharset())) {
           for (String line = reader.readLine(); line != null; line = reader.readLine()) {
             this.logger.log(LogLevel.INFO, line);
           }
         }
       }

       for (Path additionalFile: testbench.getAdditionalFiles()) {
         this.fileNames.add(additionalFile.toString());

         this.logger.log(LogLevel.INFO,
             this.logHeader + "reading the additional file: " + additionalFile);

         try (BufferedReader reader =
             Files.newBufferedReader(additionalFile, Charset.defaultCharset())) {
           for (String line = reader.readLine(); line != null; line = reader.readLine()) {
             this.logger.log(LogLevel.INFO, line);
           }
         }
       }
      }
    }
  }

  /**
   * Deletes the generated XML files.
   */
  @After
  public void deleteFiles() throws IOException {
    for (String fileName: this.fileNames) {
      this.logger.log(LogLevel.INFO, this.logHeader + "deleting the " + fileName + " file");

      Files.deleteIfExists(Paths.get(fileName));
    }
  }
}
