/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.model.efsm;

import static org.junit.Assert.*;

import org.junit.Test;

import ru.ispras.retrascope.model.efsm.examples.B04;
import ru.ispras.retrascope.model.efsm.examples.SimpleEfsm;
import ru.ispras.retrascope.util.SolverUtilTest;

/**
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public class ExampleEfsmProductTestCase extends SolverUtilTest{

  @Test
  public void testProduct() {
    Efsm efsm1 = B04.EFSM;
    Efsm efsm2 = SimpleEfsm.EFSM;

    System.out.println(efsm1.getName());
    System.out.println("States: " + efsm1.getStates().size());
    System.out.println(efsm1.getStates().toString());
    System.out.println("Transitions: " + efsm1.getTransitions().size());
    System.out.println(efsm1.getTransitions().toString());
    System.out.println();

    System.out.println(efsm2.getName());
    System.out.println("States: " + efsm2.getStates().size());
    System.out.println(efsm2.getStates().toString());
    System.out.println("Transitions: " + efsm2.getTransitions().size());
    System.out.println(efsm2.getTransitions().toString());
    System.out.println();

    Efsm product = Efsm.product(efsm1, efsm2);

    assertNotNull(product);
    assertEquals(product.getStates().size(), efsm1.getStates().size() * efsm2.getStates().size());

    System.out.println(product.getName());
    System.out.println("States: " + product.getStates().size());
    System.out.println(product.getStates().toString());
    System.out.println("Transitions: " + product.getTransitions().size());
    for (EfsmTransition item : product.getTransitions()) {
      System.out.println(item.toString());
    }
    System.out.println("Initial state: " + product.getInitialState().toString());
    System.out.println();

    Efsm product2 = Efsm.product(efsm2, efsm1);

    assertNotNull(product2);
    assertEquals(product2.getStates().size(), efsm2.getStates().size() * efsm1.getStates().size());

    System.out.println(product2.getName());
    System.out.println("States: " + product2.getStates().size());
    System.out.println("Transitions: " + product2.getTransitions().size());
    System.out.println("Initial state: " + product2.getInitialState().toString());
    System.out.println();

    Efsm product3 = Efsm.product(efsm2, efsm2, efsm2);

    assertNotNull(product3);
    assertEquals(product3.getStates().size(), efsm2.getStates().size() * efsm2.getStates().size()
        * efsm2.getStates().size());

    System.out.println(product3.getName());
    System.out.println("States: " + product3.getStates().size());
    System.out.println("Transitions: " + product3.getTransitions().size());
    System.out.println("Initial state: " + product3.getInitialState().toString());
    System.out.println();
  }

}
