package ru.ispras.retrascope.model.efsm.examples;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import ru.ispras.fortress.data.DataType;
import ru.ispras.fortress.expression.NodeOperation;
import ru.ispras.fortress.expression.NodeValue;
import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.fortress.expression.StandardOperation;
import ru.ispras.fortress.solver.constraint.Formulas;
import ru.ispras.retrascope.model.basis.Assignment;
import ru.ispras.retrascope.model.basis.Event;
import ru.ispras.retrascope.model.basis.EventType;
import ru.ispras.retrascope.model.basis.RangedVariable;
import ru.ispras.retrascope.model.basis.VariableData;
import ru.ispras.retrascope.model.basis.VariableType;
import ru.ispras.retrascope.model.efsm.Action;
import ru.ispras.retrascope.model.efsm.Guard;
import ru.ispras.retrascope.model.efsm.GuardedAction;
import ru.ispras.retrascope.model.efsm.Efsm;
import ru.ispras.retrascope.model.efsm.EfsmState;
import ru.ispras.retrascope.result.cfginterface.CfgInterface;
import ru.ispras.retrascope.result.cfginterface.CfgSignalInterface;

public class SimpleEfsm {
  public static final Efsm EFSM = new Efsm("SimpleEfsm");
  public static final CfgInterface CFG_INTERFACE = new CfgInterface();

  static {
    String clockName = "clock";

    String i0Name = "i0";
    String i1Name = "i1";
    String i2Name = "i2";

    String xName = "x";
    String yName = "y";
    String zName = "z";

    String stateName = "s";

    DataType clockDataType = DataType.BIT_VECTOR(1);

    DataType i0DataType = DataType.BIT_VECTOR(4);
    DataType i1DataType = DataType.BIT_VECTOR(1);
    DataType i2DataType = DataType.BIT_VECTOR(4);

    DataType xDataType = DataType.BIT_VECTOR(4);
    DataType yDataType = DataType.BIT_VECTOR(4);
    DataType zDataType = DataType.BIT_VECTOR(4);

    DataType stateDataType = DataType.BIT_VECTOR(4);

    SimpleEfsm.EFSM.addInput(clockName, clockDataType);

    NodeVariable clockVariable = new NodeVariable(clockName, clockDataType);

    NodeVariable i0Variable = new NodeVariable(i0Name, i0DataType);
    NodeVariable i1Variable = new NodeVariable(i1Name, i1DataType);
    NodeVariable i2Variable = new NodeVariable(i2Name, i2DataType);

    NodeVariable xVariable = new NodeVariable(xName, xDataType);
    NodeVariable yVariable = new NodeVariable(yName, yDataType);
    NodeVariable zVariable = new NodeVariable(zName, zDataType);

    NodeVariable stateVariable = new NodeVariable(stateName, stateDataType);

    clockVariable.setUserData(new VariableData(VariableType.IN));

    i0Variable.setUserData(new VariableData(VariableType.IN));
    i1Variable.setUserData(new VariableData(VariableType.IN));
    i2Variable.setUserData(new VariableData(VariableType.IN));
    
    xVariable.setUserData(new VariableData(VariableType.REG));
    yVariable.setUserData(new VariableData(VariableType.REG));
    zVariable.setUserData(new VariableData(VariableType.REG));

    stateVariable.setUserData(new VariableData(VariableType.REG));

    RangedVariable clockRangedVariable = new RangedVariable();

    RangedVariable i0RangedVariable = new RangedVariable();
    RangedVariable i1RangedVariable = new RangedVariable();
    RangedVariable i2RangedVariable = new RangedVariable();

    RangedVariable stateRangedVariable = new RangedVariable();

    clockRangedVariable.setVariable(clockVariable);

    i0RangedVariable.setVariable(i0Variable);
    i1RangedVariable.setVariable(i1Variable);
    i2RangedVariable.setVariable(i2Variable);

    stateRangedVariable.setVariable(stateVariable);

    NodeValue state0Value = new NodeValue(stateDataType.valueOf("0", 10));
    NodeValue state1Value = new NodeValue(stateDataType.valueOf("1", 10));
    NodeValue state2Value = new NodeValue(stateDataType.valueOf("2", 10));
    NodeValue state3Value = new NodeValue(stateDataType.valueOf("3", 10));
    NodeValue state4Value = new NodeValue(stateDataType.valueOf("4", 10));

    EfsmState state0 = SimpleEfsm.EFSM.createState(new NodeOperation(StandardOperation.EQ,
        stateVariable, state0Value));
    SimpleEfsm.EFSM.setInitialState(state0);
    EfsmState state1 = SimpleEfsm.EFSM.createState(new NodeOperation(StandardOperation.EQ,
        stateVariable, state1Value));
    EfsmState state2 = SimpleEfsm.EFSM.createState(new NodeOperation(StandardOperation.EQ,
        stateVariable, state2Value));
    EfsmState state3 = SimpleEfsm.EFSM.createState(new NodeOperation(StandardOperation.EQ,
        stateVariable, state3Value));
    EfsmState state4 = SimpleEfsm.EFSM.createState(new NodeOperation(StandardOperation.EQ,
        stateVariable, state4Value));

    Formulas formulasG01 = new Formulas(NodeValue.newBoolean(true));

    Formulas formulasG12 = new Formulas(new NodeOperation(StandardOperation.EQ, i1Variable,
        new NodeValue(i1DataType.valueOf("1", 10))));

    Formulas formulasG13 = new Formulas(new NodeOperation(StandardOperation.EQ, i1Variable,
        new NodeValue(i1DataType.valueOf("0", 10))));

    Formulas formulasG23 = new Formulas(NodeValue.newBoolean(true));

    Formulas formulasG34 = new Formulas();
    formulasG34.add(new NodeOperation(StandardOperation.EQ, xVariable,
        new NodeValue(xDataType.valueOf("4", 10))));
    formulasG34.add(new NodeOperation(StandardOperation.EQ, yVariable,
        new NodeValue(yDataType.valueOf("2", 10))));

    List<Assignment> action01Assignments = new ArrayList<>(2);
    action01Assignments.add(new Assignment(zVariable, i0Variable));
    action01Assignments.add(new Assignment(stateVariable, state1Value));

    List<Assignment> action12Assignments = new ArrayList<>(2);
    action12Assignments.add(new Assignment(xVariable, zVariable));
    action12Assignments.add(new Assignment(stateVariable, state2Value));

    List<Assignment> action13Assignments = new ArrayList<>(2);
    action13Assignments.add(new Assignment(xVariable, zVariable));
    action13Assignments.add(new Assignment(stateVariable, state3Value));

    List<Assignment> action23Assignments = new ArrayList<>(2);
    action23Assignments.add(new Assignment(yVariable, i2Variable));
    action23Assignments.add(new Assignment(stateVariable, state3Value));

    Action action01 = new Action(action01Assignments);
    Action action12 = new Action(action12Assignments);
    Action action13 = new Action(action13Assignments);
    Action action23 = new Action(action23Assignments);
    Action action34 = new Action(new Assignment(stateVariable, state4Value));

    Event clockEvent = new Event(clockRangedVariable, EventType.POSITIVE_EDGE);

    GuardedAction ga01 = new GuardedAction(new Guard(formulasG01.asSingleExpr()), action01);
    GuardedAction ga12 = new GuardedAction(new Guard(formulasG12.asSingleExpr()), action12);
    GuardedAction ga13 = new GuardedAction(new Guard(formulasG13.asSingleExpr()), action13);
    GuardedAction ga23 = new GuardedAction(new Guard(formulasG23.asSingleExpr()), action23);
    GuardedAction ga34 = new GuardedAction(new Guard(formulasG34.asSingleExpr()), action34);

    ga01.setClock(clockEvent);
    ga12.setClock(clockEvent);
    ga13.setClock(clockEvent);
    ga23.setClock(clockEvent);
    ga34.setClock(clockEvent);

    SimpleEfsm.EFSM.createTransition(state0, state1, ga01);
    SimpleEfsm.EFSM.createTransition(state1, state2, ga12);
    SimpleEfsm.EFSM.createTransition(state1, state3, ga13);
    SimpleEfsm.EFSM.createTransition(state2, state3, ga23);
    SimpleEfsm.EFSM.createTransition(state3, state4, ga34);

    List<Assignment> resetAssignments = new ArrayList<>(3);
    resetAssignments.add(new Assignment(xVariable, new NodeValue(xDataType.valueOf("0", 10))));
    resetAssignments.add(new Assignment(yVariable, new NodeValue(yDataType.valueOf("0", 10))));
    resetAssignments.add(new Assignment(zVariable, new NodeValue(zDataType.valueOf("0", 10))));
    resetAssignments.add(new Assignment(stateVariable, state0Value));
    Action resetAction = new Action(resetAssignments);
    Guard resetGuard = new Guard(NodeValue.newBoolean(true));
    GuardedAction resetGuardedAction = new GuardedAction(resetGuard, resetAction);
    resetGuardedAction.setClock(clockEvent);

    SimpleEfsm.EFSM.setResetGuardedAction(resetGuardedAction);
  }

  static {
    List<NodeVariable> inputs = new ArrayList<>(SimpleEfsm.EFSM.getInputDataTypes().size());

    for (Map.Entry<String,DataType> input: SimpleEfsm.EFSM.getInputDataTypes().entrySet()) {
      inputs.add(new NodeVariable(input.getKey(), input.getValue()));
    }

    List<NodeVariable> outputs = new ArrayList<>(SimpleEfsm.EFSM.getInputDataTypes().size());

    for (Map.Entry<String,DataType> output: SimpleEfsm.EFSM.getOutputDataTypes().entrySet()) {
      outputs.add(new NodeVariable(output.getKey(), output.getValue()));
    }

    CfgSignalInterface cfgSignalInterface = new CfgSignalInterface(inputs, outputs);
    SimpleEfsm.CFG_INTERFACE.addInterface(SimpleEfsm.EFSM.getName(), cfgSignalInterface);
  }

  private SimpleEfsm() {}
}
