package ru.ispras.retrascope.model.efsm.examples;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import ru.ispras.fortress.data.DataType;
import ru.ispras.fortress.expression.NodeOperation;
import ru.ispras.fortress.expression.NodeValue;
import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.fortress.expression.StandardOperation;
import ru.ispras.fortress.solver.constraint.Formulas;
import ru.ispras.retrascope.model.basis.Assignment;
import ru.ispras.retrascope.model.basis.Event;
import ru.ispras.retrascope.model.basis.EventType;
import ru.ispras.retrascope.model.basis.RangedVariable;
import ru.ispras.retrascope.model.basis.VariableData;
import ru.ispras.retrascope.model.basis.VariableType;
import ru.ispras.retrascope.model.efsm.Action;
import ru.ispras.retrascope.model.efsm.Guard;
import ru.ispras.retrascope.model.efsm.GuardedAction;
import ru.ispras.retrascope.model.efsm.Efsm;
import ru.ispras.retrascope.model.efsm.EfsmState;
import ru.ispras.retrascope.result.cfginterface.CfgInterface;
import ru.ispras.retrascope.result.cfginterface.CfgSignalInterface;

public class SimplePipelinedEfsm {
  public static final Efsm EFSM = new Efsm("SimplePipelinedEfsm");
  public static final CfgInterface CFG_INTERFACE = new CfgInterface();

  static {
    String clockName = "clock";

    String i0Name = "i0";
    String i1Name = "i1";

    String x0Name = "x0";
    String x1Name = "x1";
    String x2Name = "x2";
    String x3Name = "x3";

    String stateName = "s";

    DataType clockDataType = DataType.BIT_VECTOR(1);

    DataType i0DataType = DataType.BIT_VECTOR(4);
    DataType i1DataType = DataType.BIT_VECTOR(1);

    DataType x0DataType = DataType.BIT_VECTOR(4);
    DataType x1DataType = DataType.BIT_VECTOR(4);
    DataType x2DataType = DataType.BIT_VECTOR(4);
    DataType x3DataType = DataType.BIT_VECTOR(4);

    DataType stateDataType = DataType.BIT_VECTOR(4);

    SimplePipelinedEfsm.EFSM.addInput(clockName, clockDataType);

    NodeVariable clockVariable = new NodeVariable(clockName, clockDataType);

    NodeVariable i0Variable = new NodeVariable(i0Name, i0DataType);
    NodeVariable i1Variable = new NodeVariable(i1Name, i1DataType);

    NodeVariable x0Variable = new NodeVariable(x0Name, x0DataType);
    NodeVariable x1Variable = new NodeVariable(x1Name, x1DataType);
    NodeVariable x2Variable = new NodeVariable(x2Name, x2DataType);
    NodeVariable x3Variable = new NodeVariable(x3Name, x3DataType);

    NodeVariable stateVariable = new NodeVariable(stateName, stateDataType);

    clockVariable.setUserData(new VariableData(VariableType.IN));

    i0Variable.setUserData(new VariableData(VariableType.IN));
    i1Variable.setUserData(new VariableData(VariableType.IN));
    
    x0Variable.setUserData(new VariableData(VariableType.REG));
    x1Variable.setUserData(new VariableData(VariableType.REG));
    x2Variable.setUserData(new VariableData(VariableType.REG));
    x3Variable.setUserData(new VariableData(VariableType.REG));

    stateVariable.setUserData(new VariableData(VariableType.REG));

    RangedVariable clockRangedVariable = new RangedVariable();

    RangedVariable i0RangedVariable = new RangedVariable();
    RangedVariable i1RangedVariable = new RangedVariable();

    RangedVariable stateRangedVariable = new RangedVariable();

    clockRangedVariable.setVariable(clockVariable);

    i0RangedVariable.setVariable(i0Variable);
    i1RangedVariable.setVariable(i1Variable);

    stateRangedVariable.setVariable(stateVariable);

    NodeValue state0Value = new NodeValue(stateDataType.valueOf("0", 10));
    NodeValue state1Value = new NodeValue(stateDataType.valueOf("1", 10));
    NodeValue state2Value = new NodeValue(stateDataType.valueOf("2", 10));
    NodeValue state3Value = new NodeValue(stateDataType.valueOf("3", 10));

    EfsmState state0 = SimplePipelinedEfsm.EFSM.createState(new NodeOperation(StandardOperation.EQ,
        stateVariable, state0Value));
    SimplePipelinedEfsm.EFSM.setInitialState(state0);

    EfsmState state1 = SimplePipelinedEfsm.EFSM.createState(new NodeOperation(StandardOperation.EQ,
        stateVariable, state1Value));
    
    EfsmState state2 = SimplePipelinedEfsm.EFSM.createState(new NodeOperation(StandardOperation.EQ,
        stateVariable, state2Value));
    
    EfsmState state3 = SimplePipelinedEfsm.EFSM.createState(new NodeOperation(StandardOperation.EQ,
        stateVariable, state3Value));

    Formulas formulasG01 = new Formulas(NodeValue.newBoolean(true));

    Formulas formulasG11 = new Formulas(new NodeOperation(StandardOperation.EQ, i1Variable,
        new NodeValue(i1DataType.valueOf("0", 10))));

    Formulas formulasG12 = new Formulas();
    formulasG12.add(new NodeOperation(StandardOperation.NOTEQ, i1Variable,
        new NodeValue(i1DataType.valueOf("0", 10))));
    formulasG12.add(new NodeOperation(StandardOperation.EQ, x3Variable,
        new NodeValue(x3DataType.valueOf("4", 10))));

    Formulas formulasG13 = new Formulas();
    formulasG13.add(new NodeOperation(StandardOperation.NOTEQ, i1Variable,
        new NodeValue(i1DataType.valueOf("0", 10))));
    formulasG13.add(new NodeOperation(StandardOperation.NOTEQ, x3Variable,
        new NodeValue(x3DataType.valueOf("4", 10))));

    List<Assignment> action01Statements = new ArrayList<>(2);
    action01Statements.add(new Assignment(x0Variable, i0Variable));
    action01Statements.add(new Assignment(stateVariable, state1Value));

    ArrayList<Assignment> action11Statements = new ArrayList<>();
    action11Statements.add(new Assignment(x3Variable, x2Variable));
    action11Statements.add(new Assignment(x2Variable, x1Variable));
    action11Statements.add(new Assignment(x1Variable, x0Variable));

    Action action01 = new Action(action01Statements);
    Action action11 = new Action(action11Statements);
    Action action12 = new Action(new Assignment(stateVariable, state2Value));
    Action action13 = new Action(new Assignment(stateVariable, state3Value));

    Event clockEvent = new Event(clockRangedVariable, EventType.POSITIVE_EDGE);

    GuardedAction ga01 = new GuardedAction(new Guard(formulasG01.asSingleExpr()), action01);
    GuardedAction ga11 = new GuardedAction(new Guard(formulasG11.asSingleExpr()), action11);
    GuardedAction ga12 = new GuardedAction(new Guard(formulasG12.asSingleExpr()), action12);
    GuardedAction ga13 = new GuardedAction(new Guard(formulasG13.asSingleExpr()), action13);

    ga01.setClock(clockEvent);
    ga11.setClock(clockEvent);
    ga12.setClock(clockEvent);
    ga13.setClock(clockEvent);

    SimplePipelinedEfsm.EFSM.createTransition(state0, state1, ga01);
    SimplePipelinedEfsm.EFSM.createTransition(state1, state1, ga11);
    SimplePipelinedEfsm.EFSM.createTransition(state1, state2, ga12);
    SimplePipelinedEfsm.EFSM.createTransition(state1, state3, ga13);

    List<Assignment> resetAssignments = new ArrayList<>(3);
    resetAssignments.add(new Assignment(stateVariable, state0Value));
    resetAssignments.add(new Assignment(x0Variable, new NodeValue(x0DataType.valueOf("0", 10))));
    resetAssignments.add(new Assignment(x1Variable, new NodeValue(x1DataType.valueOf("0", 10))));
    resetAssignments.add(new Assignment(x2Variable, new NodeValue(x2DataType.valueOf("0", 10))));
    resetAssignments.add(new Assignment(x3Variable, new NodeValue(x3DataType.valueOf("0", 10))));
    Action resetAction = new Action(resetAssignments);
    Guard resetGuard = new Guard(NodeValue.newBoolean(true));
    GuardedAction resetGuardedAction = new GuardedAction(resetGuard, resetAction);
    resetGuardedAction.setClock(clockEvent);
    SimplePipelinedEfsm.EFSM.setResetGuardedAction(resetGuardedAction);
  }

  static {
    List<NodeVariable> inputs =
        new ArrayList<>(SimplePipelinedEfsm.EFSM.getInputDataTypes().size());

    for (Map.Entry<String,DataType> input:
        SimplePipelinedEfsm.EFSM.getInputDataTypes().entrySet()) {
      inputs.add(new NodeVariable(input.getKey(), input.getValue()));
    }

    List<NodeVariable> outputs =
        new ArrayList<>(SimplePipelinedEfsm.EFSM.getInputDataTypes().size());

    for (Map.Entry<String,DataType> output:
        SimplePipelinedEfsm.EFSM.getOutputDataTypes().entrySet()) {
      outputs.add(new NodeVariable(output.getKey(), output.getValue()));
    }

    CfgSignalInterface cfgSignalInterface = new CfgSignalInterface(inputs, outputs);
    SimplePipelinedEfsm.CFG_INTERFACE.addInterface(SimplePipelinedEfsm.EFSM.getName(),
        cfgSignalInterface);
  }

  private SimplePipelinedEfsm() {}
}
