/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.util;

import ru.ispras.retrascope.VerilogFiles;
import ru.ispras.retrascope.VhdlFiles;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * Utility for testing at module-level.
 * Can be applied for HDL modules that languages are supported by the project.
 * .
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class HdlUtilTest {

  /**
   * Runs tool on HDL modules with the specified target.
   *
   * @param target tool target
   */
  public static void runHdl(String target) {
    runVhdl(target);
    runVerilog(target);
  }

  /**
   * Runs tool on HDL modules with the specified target and options.
   *
   * @param target tool target
   * @param additionalOptions additional options
   */
  public static void runHdl(String target, String[] additionalOptions) {
    runVhdl(target, additionalOptions);
    runVerilog(target, additionalOptions);
  }

  /**
   * Runs tool on VHDL modules with the specified target.
   *
   * @param target tool target
   */
  public static void runVhdl(String target) {

    // get VHDL projects
    Map<String, Map<String, String>> vhdlFiles = VhdlFiles.getFiles();

    // run Retrascope on each module of each VHDL project
    for (String project : vhdlFiles.keySet()) {
      Map<String, String> projectMap = vhdlFiles.get(project);

      for (String fileName : projectMap.keySet()) {
        Map<String, String> onlyFile = new LinkedHashMap<>();
        onlyFile.put(fileName, projectMap.get(fileName));
        VhdlUtilTest.runVhdl(onlyFile, target);
      }
    }
  }

  /**
   * Runs tool on VHDL modules with the specified target and options.
   *
   * @param target tool target
   * @param additionalOptions additional options
   */
  public static void runVhdl(String target, String[] additionalOptions) {

    // get VHDL projects
    Map<String, Map<String, String>> vhdlFiles = VhdlFiles.getFiles();

    // run Retrascope on each module of each VHDL project
    for (String project : vhdlFiles.keySet()) {
      Map<String, String> projectMap = vhdlFiles.get(project);

      for (String fileName : projectMap.keySet()) {
        Map<String, String> onlyFile = new LinkedHashMap<>();
        onlyFile.put(fileName, projectMap.get(fileName));
        VhdlUtilTest.runVhdl(onlyFile, target, additionalOptions);
      }
    }
  }

  /**
   * Runs tool on Verilog modules with the specified target.
   *
   * @param target tool target
   */
  public static void runVerilog(String target) {
    // get Verilog projects
    Map<String, Map<String, String>> verilogFiles = VerilogFiles.getFiles();

    // run Retrascope on each module of each Verilog project
    for (String project : verilogFiles.keySet()) {
      Map<String, String> projectMap = verilogFiles.get(project);

      for (String fileName : projectMap.keySet()) {
        Map<String, String> onlyFile = new LinkedHashMap<>();
        onlyFile.put(fileName, projectMap.get(fileName));
        VerilogUtilTest.runVerilog(onlyFile, target);
      }
    }
  }

  /**
   * Runs tool on Verilog modules with the specified target and options.
   *
   * @param target tool target
   * @param additionalOptions additional options
   */
  public static void runVerilog(String target, String[] additionalOptions) {
    // get Verilog projects
    Map<String, Map<String, String>> verilogFiles = VerilogFiles.getFiles();

    // run Retrascope on each module of each Verilog project
    for (String project : verilogFiles.keySet()) {
      Map<String, String> projectMap = verilogFiles.get(project);

      for (String fileName : projectMap.keySet()) {
        Map<String, String> onlyFile = new LinkedHashMap<>();
        onlyFile.put(fileName, projectMap.get(fileName));
        VerilogUtilTest.runVerilog(onlyFile, target, additionalOptions);
      }
    }
  }
}
