/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.util;

import java.io.File;
import java.util.Map;

import org.apache.commons.lang.ArrayUtils;

import ru.ispras.retrascope.Retrascope;

/**
 * Utilities to use Verilog files for tests
 * 
 * @author <a href="mailto:chupilko@ispras.ru">Mikhail Chupilko</a>
 */

public class VerilogUtilTest {

  /**
   * 
   * 
   * @param map Verilog files for processing
   */
  public static void runVerilog(Map<String, String> map, String target) {
    runVerilog(map, target, "");
  }

  /**
   * 
   * 
   * @param map file map for processing
   * @param target for command line
   * @param additionalOption an option to run the test
   */
  public static void runVerilog(Map<String, String> map, String target, String additionalOption) {
      String[] fileNames = getFileNames(map);
      
      runRetrascope(fileNames, createCommandLine(target, additionalOption));
  }

  /**
   * Runs Retrascope on Verilog file with the specified options.
   *
   * @param map map of files to be processed
   * @param target value of command line target option
   * @param additionalOptions additional command line options
   */
  public static void runVerilog(
      Map<String, String> map,
      String target,
      String[] additionalOptions) {
    String[] fileNames = getFileNames(map);

    runRetrascope(fileNames, createCommandLine(target, additionalOptions));
  }
  
  /**
   * Create a list of files for the command line
   * 
   * @param map Verilog files for processing
   */
  private static String[] getFileNames(Map<String, String> map){
    String[] fileNames = new String[map.size()];

    int i = 0;
    for (String fileName : map.keySet()){
      fileNames[i] = (new File(fileName)).getAbsolutePath();
      i++;
    }
  
    return fileNames;
  }

  /**
   * Create command line
   * 
   * @param target for Retrascope
   * @param additionalOption an option to run the test
   */
  private static String[] createCommandLine(String target, String additionalOption) {
    return new String[] {"--" + Retrascope.TARGET_PARAM.getLongName(), target, additionalOption};
  }

  /**
   * Creates command line.
   *
   * @param target value of command line target option
   * @param additionalOptions additional command line options
   * @return command line arguments
   */
  private static String[] createCommandLine(String target, String[] additionalOptions) {
    return (String[]) ArrayUtils.addAll(
        new String[] {"--" + Retrascope.TARGET_PARAM.getLongName(), target},
        additionalOptions);
  }
  
  /**
   * Run Retrascope
   * 
   * @param fileNames name of Verilog files for processing
   * @param options command line
   */
  private static void runRetrascope(String[] fileNames, String[] options) {
    Retrascope.main((String[]) ArrayUtils.addAll(fileNames, options));
  }
}
